from odoo import api, fields, models
import re


class HelpdeskTicket(models.Model):
    _inherit = "helpdesk.ticket"

    email_cc = fields.Char("Cc", help="Carbon copy message recipients")

    def clean_email_cc(self, email_cc):
        # The pattern looks for anything XXXX@XXXX.XXX
        patron = r"[\w\.-]+@[\w\.-]+"

        # Find all matches using findall instead of a loop
        matches = re.findall(patron, email_cc)

        # Join all matches with comma separator
        cleaned_cc = ",".join(matches)

        return cleaned_cc

    @api.model
    def message_new(self, msg, custom_values=None):
        """Override message_new from mail gateway so we can set correct
        default values.
        """
        if custom_values is None:
            custom_values = {}

        defaults = {
            "email_cc": self.clean_email_cc(msg.get("cc", "")),
        }
        defaults.update(custom_values)

        # Write default values coming from msg
        ticket = super().message_new(msg, custom_values=defaults)

        return ticket



# Copyright 2023-SomItCoop SCCL(<https://gitlab.com/somitcoop>)
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl.html).
{
    "version": "12.0.0.1.1",
    "name": "Helpdesk with CC and TO text fields",
    "depends": [
        "helpdesk_mgmt",
        "mail_cc_and_to_text",
    ],
    "author": """
        Som It Cooperatiu SCCL,
        Som Connexió SCCL,
        Odoo Community Association (OCA)
    """,
    "category": "Auth",
    "website": "https://gitlab.com/somitcoop/erp-research/odoo-helpdesk",
    "license": "AGPL-3",
    "summary": """
        Helpdesk with CC and TO text fields without res_partner model dependency".
    """,
    "data": ["views/helpdesk_ticket_view.xml"],
    "demo": [],
    "application": False,
    "installable": True,
}



from odoo.tests.common import TransactionCase, tagged
from odoo.tools import mute_logger


@tagged("post_install", "-at_install", "helpdesk")
class TestHelpdeskTicket(TransactionCase):
    def setUp(self):
        super(TestHelpdeskTicket, self).setUp()
        self.helpdesk_ticket = self.env["helpdesk.ticket"]
        self.author_id = self.env["res.partner"].search([], limit=1)

    def test_clean_email_cc(self):
        ticket = self.helpdesk_ticket

        # Test case with valid cc emails
        cc_emails = "<test1@example.com>,'test2' <test2@example.com>,test3@example.com"
        cleaned_cc = ticket.clean_email_cc(cc_emails)
        self.assertEqual(
            cleaned_cc, "test1@example.com,test2@example.com,test3@example.com"
        )

        # Test case with invalid cc emails
        cc_emails_invalid = (
            "'test1' <test1@example.com>,'test2' <test2@example.com>,invalidemail"
        )
        cleaned_cc_invalid = ticket.clean_email_cc(cc_emails_invalid)
        self.assertEqual(cleaned_cc_invalid, "test1@example.com,test2@example.com")

        # Test case with empty cc emails
        cc_emails_empty = ""
        cleaned_cc_empty = ticket.clean_email_cc(cc_emails_empty)
        self.assertEqual(cleaned_cc_empty, "")

    @mute_logger("odoo.sql_db")
    def test_message_new(self):
        ticket = self.helpdesk_ticket

        message = {
            "to": "test@example.com",
            "cc": "",
            "subject": "prueba",
            "body": "prueba",
            "author_id": self.author_id.id,
        }
        ticket_no_custom = ticket.with_context(message_new_test=True).message_new(
            message
        )
        self.assertEqual(ticket_no_custom.email_cc, "")

        # Test case with invalid cc
        message_invalid_cc = {
            "to": "test@example.com",
            "cc": "invalidemail",
            "subject": "prueba",
            "body": "prueba",
            "author_id": self.author_id.id,
        }
        ticket.with_context(message_new_test=True).message_new(
            message_invalid_cc
        )
